/**
  @file videocapture_basic.cpp
  @brief A very basic sample for using VideoCapture and VideoWriter
  @author PkLab.net
  @date Aug 24, 2016
*/

#include <opencv2/core.hpp>
#include <opencv2/videoio.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc.hpp>
#include <iostream>
#include <stdio.h>

std::string keys =
    "{ help  h     | | Print help message. }"
    "{ device      |  4 | camera device number, 1 for CSI Camera with ISP, 4 for UVC in common case. }";

using namespace cv;
using namespace std;

int main(int argc, char** argv)
{
    Mat frame;
    Mat frame_rgb24;

    CommandLineParser parser(argc, argv, keys);
    parser.about("Use this script to run edge detection using OpenCV.");
    if (parser.has("help"))
    {
        parser.printMessage();
        return 0;
    }

    //--- INITIALIZE VIDEOCAPTURE
    VideoCapture cap;
    // open the default camera using default API
    // cap.open(0);
    // OR advance usage: select any API backend
    int deviceID = parser.get<int>("device");
    int apiID = cv::CAP_V4L2;
    // open selected camera using selected API
    cap.open(deviceID, apiID);
    // check if we succeeded
    if (!cap.isOpened()) {
        cerr << "ERROR! Unable to open camera\n";
        return -1;
    }

    //FIXME: Auto convert property failed in CSI camera(NV12 format).
    if (parser.get<int>("device") == 1)
        cap.set(CAP_PROP_CONVERT_RGB , false);

    //--- GRAB AND WRITE LOOP
    cout << "Start grabbing" << endl
        << "Press any key to terminate" << endl;
    for (;;)
    {
        // wait for a new frame from camera and store it into 'frame'
        if (parser.get<int>("device") == 1)
        {
            //FIXME: Workaround by cvtColor from nv12 to bgr.
            cap.read(frame_rgb24);
            cvtColor(frame_rgb24, frame, cv::COLOR_YUV2BGR_NV12);
        } else {
            cap.read(frame);
        }

        // check if we succeeded
        if (frame.empty()) {
            cerr << "ERROR! blank frame grabbed\n";
            break;
        }
        // show live and wait for a key with timeout long enough to show images
        imshow("Live", frame);
        if (waitKey(5) >= 0)
            break;
    }
    // the camera will be deinitialized automatically in VideoCapture destructor
    return 0;
}
